// Copyright: Apache2 license
package test.teneo.inheritance;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Properties;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.teneo.hibernate.HbDataStore;
import org.eclipse.emf.teneo.hibernate.HbHelper;
import org.eclipse.emf.teneo.hibernate.resource.HibernateResource;
import org.hibernate.cfg.Environment;
import org.hibernate.dialect.HSQLDialect;

import testinheritance.NameValuePair;
import testinheritance.SomeResource;
import testinheritance.TestinheritanceFactory;
import testinheritance.TestinheritancePackage;

public class InheritanceTests extends TestCase {
	private static final String DBNAME="InheritanceTestsDB";
	
    private static Properties getHibernateProperties(String dbName) {
        final Properties props = new Properties(); 
        props.setProperty(Environment.DRIVER, "org.hsqldb.jdbcDriver");
        props.setProperty(Environment.USER, "sa");
        props.setProperty(Environment.URL, "jdbc:hsqldb:file:" + dbName + ";shutdown=true");
        props.setProperty(Environment.PASS, "");
        props.setProperty(Environment.DIALECT, HSQLDialect.class.getName());
        props.setProperty(Environment.CURRENT_SESSION_CONTEXT_CLASS, "thread");
        return props;
    }

    public static void removeDBFiles(String dbname) {
        for (String dbFile: new File(".").list()) {
            File f = new File(dbFile);
            if (f.getName().startsWith(dbname)) {
                if (!f.delete()) {
                    throw new RuntimeException("Lock still held on DB file" + f.getName());
                }
            }
        }
    }

    /* (non-Javadoc)
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
        removeDBFiles(DBNAME);
	}

	private Resource createResource(String dbname) {
		return createResource(dbname, null);
	}
    private HbDataStore createHbDataStore(String dbname) {
		Properties props = getHibernateProperties(dbname);
		EPackage[]packages = new EPackage[]{TestinheritancePackage.eINSTANCE};
		HbDataStore hbds = HbHelper.INSTANCE.createRegisterDataStore(dbname);
        hbds.setEPackages(packages);
        // set the relational database to use and other properties
        hbds.setHibernateProperties(props);
        hbds.setPersistenceProperties(props);
        hbds.initialize();
        return hbds;
	}
    private Resource createResource(String dbname, String queryString) {
		String uriStr = "hibernate://?" + HibernateResource.DS_NAME_PARAM + "=" + dbname;
		if (queryString != null) {
			uriStr += queryString;
		}
		final URI uri = URI.createURI(uriStr);
        return new HibernateResource(uri);
	}
    
    @SuppressWarnings("unchecked")
	public void testSavingAuthorizables() throws IOException {
	    HbDataStore store = createHbDataStore(DBNAME);
		Resource res = null;
		try {
			res = createResource(DBNAME);
			res.load(Collections.EMPTY_MAP);
	        SomeResource someResource = TestinheritanceFactory.eINSTANCE.createSomeResource();
	        someResource.setAnotherProperty(42);
	        someResource.setProperty("foo");
	        NameValuePair nv = TestinheritanceFactory.eINSTANCE.createNameValuePair();
	        nv.setName("foo1");
	        nv.setValue("bar");
	        someResource.getNameValuePairs().add(nv);
			res.getContents().add(someResource);
			res.save(null);
		} finally {
			if (res != null) {
				res.unload();
			}
			store.close();
		}
	}

    @SuppressWarnings("unchecked")
	public void testUpdatingAuthorizables() throws IOException {
		SomeResource someResource;
	    HbDataStore store = createHbDataStore(DBNAME);
	    Resource res = null;
		try {
			res = createResource(DBNAME);
			res.load(Collections.EMPTY_MAP);
			someResource = TestinheritanceFactory.eINSTANCE.createSomeResource();
	        someResource.setAnotherProperty(13);
	        someResource.setProperty("baz");
			res.getContents().add(someResource);
			res.save(null);
		} finally {
			if (res != null) {
				res.unload();
			}
		}
		res = createResource(DBNAME, "&query1=FROM SomeResource");
		try {
			res.load(Collections.EMPTY_MAP);
			Assert.assertNotNull(res.getContents());
			Assert.assertEquals(1, res.getContents().size());
	        someResource = (SomeResource)res.getContents().get(0);
	        Assert.assertNotNull(someResource);
	        NameValuePair nv = TestinheritanceFactory.eINSTANCE.createNameValuePair();
	        nv.setName("foo1");
	        nv.setValue("bar");
	        someResource.getNameValuePairs().add(nv);
			res.save(null);
		} finally {
			if (res != null) {
				res.unload();
			}
			store.close();
		}
	}
}
